Struct moonshine_kind::Instance

source ·
pub struct Instance<T: Kind>(/* private fields */);
Represents an Entity of Kind T.

Instance<Any> is functionally equivalent to an entity.

Usage

An Instance<T> can be used to access entities in a “kind-safe” manner to improve safety and readability.

This type is designed to behave exactly like an Entity.

This means you may use it as a Query parameter, pass it to Commands to access InstanceCommands<T>, or store it as a type-safe reference to an Entity.

Note that an Instance<T> has 'static lifetime and does not contain any Component data. It only contains type information.

Example


#[derive(Component)]
struct Apple;

#[derive(Component)]
struct Orange;

struct Fruit;

impl Kind for Fruit {
    type Filter = Or<(With<Apple>, With<Orange>)>;
}

#[derive(Resource, Deref, DerefMut)]
struct FruitBasket(Vec<Instance<Fruit>>);

fn collect_fruits(mut basket: ResMut<FruitBasket>, fruits: Query<Instance<Fruit>>) {
    for fruit in fruits.iter() {
        println!("{fruit:?}");
        basket.push(fruit);
    }
}

Implementations

source

impl<T: Kind> Instance<T>

source

pub const PLACEHOLDER: Self = _

Same as Entity::PLACEHOLDER, but for an Instance<T>.

source

pub unsafe fn from_entity_unchecked(entity: Entity) -> Self

Creates a new instance of kind T from some Entity.

Usage

This function is useful when you know an Entity is of a specific kind and you need an Instance<T> with no way to validate it.

See Instance::from_entity for a safer alternative.

Safety

Assumes entity is a valid instance of kind T.

Example

#[derive(Component)]
struct Apple;

fn init_apple(entity: Entity, commands: &mut Commands) -> Instance<Apple> {
    commands.entity(entity).insert(Apple);
    // SAFE: `entity` will be a valid instance of `Apple`.
    unsafe { Instance::from_entity_unchecked(entity) }
}
source

pub fn entity(&self) -> Entity

Returns the Entity of this instance.

source

pub fn cast_into<U: Kind>(self) -> Instance<U>
where T: CastInto<U>,

Converts this instance into an instance of another kind Kind U.

Usage

A kind T is safety convertible to another kind U if T implements CastInto<U>.

See [kind] macro for usage examples.

source

pub fn cast_into_any(self) -> Instance<Any>

Converts this instance into an instance of Kind Any.

Usage

Any Instance<T> can be safely cast into an Instance<Any> using this function.

source

pub unsafe fn cast_into_unchecked<U: Kind>(self) -> Instance<U>

Converts this instance into an instance of another kind Kind U without any validation.

Usage

This function is useful when you know an Instance<T> is convertible to a specific type and you need an Instance<U> with no way to validate it.

Always prefer to explicitly declare safe casts using [kind] macro and use Instance::cast_into instead of this.

Safety

Assumes this instance is also a valid Instance<U>.

source

impl<T: Component> Instance<T>

source

pub fn from_entity(entity: EntityRef<'_>) -> Option<Self>

Creates a new instance of kind T from some EntityRef if the entity has a Component of type T.

Methods from Deref<Target = Entity>

source

pub const PLACEHOLDER: Entity = _