Struct moonshine_kind::InstanceRef

source ·
pub struct InstanceRef<'a, T: Component> { /* private fields */ }
A QueryData item which represents a reference to an Instance<T> and its associated Component.

Usage

If a Kind is also a component, it is often convenient to access the instance and component data together. This type is designed to make these queries more ergonomic.

You may use this type as either a Query parameter, or access it from an EntityRef.

Example


#[derive(Component)]
struct Apple {
    freshness: f32,
}

impl Apple {
    fn is_fresh(&self) -> bool {
        self.freshness >= 0.5
    }
}

// Query Access:
fn fresh_apples(query: Query<InstanceRef<Apple>>) -> Vec<Instance<Apple>> {
    query.iter()
        .filter_map(|apple| apple.is_fresh().then_some(apple.instance()))
        .collect()
}

// Entity Access:
fn fresh_apples_world<'a>(world: &'a World) -> Vec<InstanceRef<'a, Apple>> {
   world.iter_entities()
        .filter_map(|entity| InstanceRef::from_entity(entity))
        .collect()
}

Implementations

source

impl<'a, T: Component> InstanceRef<'a, T>

source

pub fn from_entity(entity: EntityRef<'a>) -> Option<Self>

Creates a new InstanceRef<T> from an EntityRef if it contains a given Component of type T.

source

pub fn entity(&self) -> Entity

Returns the associated Entity.

source

pub fn instance(&self) -> Instance<T>

Returns the associated Instance<T>.